module GemsGenerator
  module Grape
    module Generators
      class InitGenerator < ::Rails::Generators::NamedBase
        source_root File.expand_path("../templates", __FILE__)

        desc "Add relevant gems: grape，grape_logging, grape_on_rails_routes, grape-swagger"
        def add_gems
          gem "grape", '~>0.15.0'
          gem "grape_logging", git: "git@github.com:aserafin/grape_logging.git"
          gem "grape_on_rails_routes"
          gem_group :development, :staging, :test do
            gem "grape-swagger"
            gem "grape-swagger-rails", git: "git@github.com:ruby-grape/grape-swagger-rails.git"
          end
          gem 'rack-attack'
        end

        desc "Create BaseApi "
        def copy_base_api_rb
          template "base_api.rb.erb", File.join('app/api/',"#{file_name}_api.rb")
        end

        desc "Create Concerns File"
        def copy_concerns_file
          api_exception_handler_path = "concerns/api_exception_handler.rb"
          template api_exception_handler_path, File.join('app/api/', api_exception_handler_path)

          document_handler_path = "concerns/document_handler.rb"
          template document_handler_path, File.join('app/api/', document_handler_path)

          ping_pong_path = "concerns/ping_pong.rb"
          template ping_pong_path, File.join('app/api/', ping_pong_path)

          request_logger_path = "concerns/request_logger.rb"
          template request_logger_path, File.join('app/api/', request_logger_path)
        end

        desc "Add Grape Routes"
        def insert_grape_routes
          route "mount #{class_name}API => '/'"
        end

        desc "insert Grape Swagger Routes"
        def insert_grape_swagger_routes
          route "mount GrapeSwaggerRails::Engine => '/swagger' unless Rails.env.production?"
        end

        desc "Create Swagger Config File" 
        def copy_swagger_file
          swagger_path = "config/initializers/swagger.rb"
          template "initializers/swagger.rb", swagger_path
        end

        desc "Create ApiError Lib File"
        def cope_api_error_file
          template "lib/api_error.rb", "lib/api_error.rb"
          template "locales/api_error/en.yml", "config/locales/api_error/en.yml"
        end

        desc "Rack Attack config file"
        def create_rack_attack_rb
          template "initializers/rack-attack.rb", "config/initializers/rack-attack.rb"
        end

        desc "Insert auload_paths and eager_load_paths to application" 
        def insert_application_rb
          inject_into_file "config/application.rb", after: "Rails::Application\n" do
            "    config.autoload_paths << Rails.root.join('lib')\n" +
            "    config.eager_load_paths << Rails.root.join('lib')\n"
          end


          inject_into_file "config/application.rb", after: "Rails::Application\n" do
            "    config.i18n.load_path += Dir[Rails.root.join('config', 'locales', '**/*.{rb,yml}').to_s]\n" +
            "    config.i18n.available_locales = %w(en zh-CN kr ja)\n" +
            "    config.i18n.fallbacks = [:en]\n"
          end

        end
  
      end
    end
  end
end
